@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import argparse
 import contextlib
 import io
@@ -8,12 +10,9 @@ from contextlib import redirect_stderr
 from copy import deepcopy
 from pathlib import Path
 from typing import Generator
-from typing import List
 from typing import Match
 from typing import NamedTuple
-from typing import Optional
 from typing import Sequence
-from typing import Tuple
 
 import pyupgrade._main
 
@@ -24,11 +23,22 @@ MD_RE = re.compile(
     r'(?P<after>^(?P=indent)```\s*$)',
     re.DOTALL | re.MULTILINE,
 )
-PY_LANGS = '(python|py|sage|python3|py3|numpy)'
+MD_PYCON_RE = re.compile(
+    r'(?P<before>^(?P<indent> *)```\s*pycon\n)'
+    r'(?P<code>.*?)'
+    r'(?P<after>^(?P=indent)```.*$)',
+    re.DOTALL | re.MULTILINE,
+)
+RST_PY_LANGS = frozenset(('python', 'py', 'sage', 'python3', 'py3', 'numpy'))
 BLOCK_TYPES = '(code|code-block|sourcecode|ipython)'
+DOCTEST_TYPES = '(testsetup|testcleanup|testcode)'
 RST_RE = re.compile(
     rf'(?P<before>'
-    rf'^(?P<indent> *)\.\. (jupyter-execute::|{BLOCK_TYPES}:: {PY_LANGS})\n'
+    rf'^(?P<indent> *)\.\. ('
+    rf'jupyter-execute::|'
+    rf'{BLOCK_TYPES}:: (?P<lang>\w+)|'
+    rf'{DOCTEST_TYPES}::.*'
+    rf')\n'
     rf'((?P=indent) +:.*\n)*'
     rf'\n*'
     rf')'
@@ -37,7 +47,7 @@ RST_RE = re.compile(
 )
 RST_PYCON_RE = re.compile(
     r'(?P<before>'
-    r'(?P<indent> *)\.\. (code|code-block):: pycon\n'
+    r'(?P<indent> *)\.\. ((code|code-block):: pycon|doctest::.*)\n'
     r'((?P=indent) +:.*\n)*'
     r'\n*'
     r')'
@@ -55,6 +65,12 @@ LATEX_RE = re.compile(
     r'(?P<after>^(?P=indent)\\end{minted}\s*$)',
     re.DOTALL | re.MULTILINE,
 )
+LATEX_PYCON_RE = re.compile(
+    r'(?P<before>^(?P<indent> *)\\begin{minted}{pycon}\n)'
+    r'(?P<code>.*?)'
+    r'(?P<after>^(?P=indent)\\end{minted}\s*$)',
+    re.DOTALL | re.MULTILINE,
+)
 PYTHONTEX_LANG = r'(?P<lang>pyblock|pycode|pyconsole|pyverbatim)'
 PYTHONTEX_RE = re.compile(
     rf'(?P<before>^(?P<indent> *)\\begin{{{PYTHONTEX_LANG}}}\n)'
@@ -89,8 +105,8 @@ def _format_str(contents_text: str, args: argparse.Namespace) -> str:
 
 def format_str(
         src: str, args: argparse.Namespace
-) -> Tuple[str, Sequence[CodeBlockError]]:
-    errors: List[CodeBlockError] = []
+) -> tuple[str, Sequence[CodeBlockError]]:
+    errors: list[CodeBlockError] = []
 
     @contextlib.contextmanager
     def _collect_error(match: Match[str]) -> Generator[None, None, None]:
@@ -107,6 +123,9 @@ def format_str(
         return f'{match["before"]}{code}{match["after"]}'
 
     def _rst_match(match: Match[str]) -> str:
+        lang = match['lang']
+        if lang is not None and lang not in RST_PY_LANGS:
+            return match[0]
         min_indent = min(INDENT_RE.findall(match['code']))
         trailing_ws_match = TRAILING_NL_RE.search(match['code'])
         assert trailing_ws_match
@@ -117,7 +136,7 @@ def format_str(
         code = textwrap.indent(code, min_indent)
         return f'{match["before"]}{code.rstrip()}{trailing_ws}'
 
-    def _rst_pycon_match(match: Match[str]) -> str:
+    def _pycon_match(match: Match[str]) -> str:
         code = ''
         fragment = None
 
@@ -152,7 +171,15 @@ def format_str(
                 else:
                     code += orig_line[indentation:] + '\n'
         finish_fragment()
+        return code
 
+    def _md_pycon_match(match: Match[str]) -> str:
+        code = _pycon_match(match)
+        code = textwrap.indent(code, match['indent'])
+        return f'{match["before"]}{code}{match["after"]}'
+
+    def _rst_pycon_match(match: Match[str]) -> str:
+        code = _pycon_match(match)
         min_indent = min(INDENT_RE.findall(match['code']))
         code = textwrap.indent(code, min_indent)
         return f'{match["before"]}{code}'
@@ -164,10 +191,17 @@ def format_str(
         code = textwrap.indent(code, match['indent'])
         return f'{match["before"]}{code}{match["after"]}'
 
+    def _latex_pycon_match(match: Match[str]) -> str:
+        code = _pycon_match(match)
+        code = textwrap.indent(code, match['indent'])
+        return f'{match["before"]}{code}{match["after"]}'
+
     src = MD_RE.sub(_md_match, src)
+    src = MD_PYCON_RE.sub(_md_pycon_match, src)
     src = RST_RE.sub(_rst_match, src)
     src = RST_PYCON_RE.sub(_rst_pycon_match, src)
     src = LATEX_RE.sub(_latex_match, src)
+    src = LATEX_PYCON_RE.sub(_latex_pycon_match, src)
     src = PYTHONTEX_RE.sub(_latex_match, src)
     return src, errors
 
@@ -192,7 +226,7 @@ def format_file(
         return 0
 
 
-def main(argv: Optional[Sequence[str]] = None) -> int:
+def main(argv: Sequence[str] | None = None) -> int:
     parser = argparse.ArgumentParser()
     parser.add_argument('--exit-zero-even-if-changed', action='store_true')
     parser.add_argument('--keep-percent-format', action='store_true')
@@ -234,4 +268,4 @@ def main(argv: Optional[Sequence[str]] = None) -> int:
 
 
 if __name__ == '__main__':
-    exit(main())
+    raise SystemExit(main())
